/* 
 * libxmlconfig - XML Config File Access Library
 * * Copyright (C) 2005 by Pengutronix e.K. <bugs@pengutronix.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */ 

#include <math.h>
#include <errno.h>
#include <string.h>
#include "include/xmlconfig.h"

#define TABLE_SIZE (int)4
#define STR_LEN (int)20

#define CONFIGFILE "test.xml"

int main(int argc, char **argv) {

        xmlconfig_t *cfg;
	int res;
	unsigned long ulBuf;
	long lBuf;
	double dBuf; 
	unsigned long ltable[TABLE_SIZE];
	double dtable[TABLE_SIZE];
	char cBuf[STR_LEN]; 
	char ctable[TABLE_SIZE][STR_LEN];

	/* parse document */
	printf("parsing document                                          ");
        cfg = xmlconfig_init(CONFIGFILE);
        if (!cfg) {
                printf("FAILED: could not open XML file <%s>\n", CONFIGFILE);
                exit(EXIT_FAILURE);
        }

	printf("OK\n");

	/* test: one hex value */
	res = xmlconfig_get_ulong(cfg, "//config/section[@id='sec1']/topic/val1", &ulBuf, 1); 
	printf("xpath=//config/section[@id='sec1']/topic/val1             ");
	if ((ulBuf != 10) || (res != 1)) {
		printf("FAILED: expected 10, got %lu\n", ulBuf);
		exit(EXIT_FAILURE);
	}

	printf("OK\n");

	/* test: attribute contains negative value, error*/
	res = xmlconfig_get_ulong(cfg, "//config/section[@id='sec1']/@val", &ulBuf, 1); 
	printf("xpath=//config/section[@id='sec1']/@val                   ");
	if ((res != -ERANGE)) {
		printf("FAILED: expected retval -ERANGE, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");

	/* test: attribute contains negative value */
	res = xmlconfig_get_long(cfg, "//config/section[@id='sec1']/@val", &lBuf, 1); 
	printf("xpath=//config/section[@id='sec1']/@val                   ");
	if ((lBuf != -55) || (res != 1)) {
		printf("FAILED: expected retval -55, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");

	/* test: multi line match, should barf */
	res = xmlconfig_get_ulong(cfg, "//config", &ulBuf, 1); 
	printf("xpath=//config                                            ");
	if ((res != -ERANGE)) {
		printf("FAILED: expected retval -ERANGE, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");

	/* test: normal non-negative value */
	res = xmlconfig_get_ulong(cfg, "//config/section[@id='sec1']/topic/val2", &ulBuf, 1); 
	printf("xpath=//config/section[@id='sec1']/topic/val2             ");
	if ((ulBuf != 112) || (res != 1)) {
		printf("FAILED: expected 10, got %lu\n", ulBuf);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");

	/* test: get multi-match list */
	res = xmlconfig_get_ulong(cfg, "//config/section[@id='sec2']/topic/val", &ltable[0], TABLE_SIZE); 
	printf("xpath=//config/section[@id='sec2']/topic/val              ");
	if (res != 6) {
		printf("FAILED: expected 6 matches, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	if (ltable[0] != 1) {
		printf("FAILED: expected 1, got %lu\n", ltable[0]);
		exit(EXIT_FAILURE);
	}
	if (ltable[1] != 2) {
		printf("FAILED: expected 2, got %lu\n", ltable[1]);
		exit(EXIT_FAILURE);
	}
	if (ltable[2] != 3) {
		printf("FAILED: expected 3, got %lu\n", ltable[2]);
		exit(EXIT_FAILURE);
	}
	if (ltable[3] != 4) {
		printf("FAILED: expected 4, got %lu\n", ltable[3]);
		exit(EXIT_FAILURE);
	}

	printf("OK\n");

	/* test: get floating point */
	res = xmlconfig_get_double(cfg, "//config/section[@id='sec4']/topic/val", &dBuf, 1);
	printf("xpath=//config/section[@id='sec4']/topic/val              ");
	if (res != 1) {
		printf("FAILED: expected 1 match, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	if ((int)(dBuf*100) != 314) {
		printf("FAILED: expected 3.14, got %f\n", dBuf);
		exit(EXIT_FAILURE);
	}

	printf("OK\n");
	
	/* test: get floating point */
	res = xmlconfig_get_double(cfg, "//config/section[@id='sec5']/topic/val", &dtable[0], TABLE_SIZE);
	printf("xpath=//config/section[@id='sec5']/topic/val              ");
	if (res != 6) {
		printf("FAILED: expected 6 matches, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	if (lrint(dtable[0]*10) != 11) {
		printf("FAILED: expected 1.1, got %f\n", dtable[0]);
		exit(EXIT_FAILURE);
	}
	if (lrint(dtable[1]*10) != 22) {
		printf("FAILED: expected 2.2, got %f\n", dtable[1]);
		exit(EXIT_FAILURE);
	}
	if (lrint(dtable[2]*10) != 33) {
		printf("FAILED: expected 3.3, got %f\n", dtable[2]);
		exit(EXIT_FAILURE);
	}
	if (lrint(dtable[3]*10) != 44) {
		printf("FAILED: expected 4.4, got %f\n", dtable[3]);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");
	
	/* test: get one string */
	res = xmlconfig_get_string(cfg, "//config/section[@id='sec6']/topic/val", &cBuf[0], STR_LEN, 1);
	printf("xpath=//config/section[@id='sec6']/topic/val              ");
	if (res != 1) {
		printf("FAILED: expected 1 match, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	if (strcmp(cBuf,"erich") != 0) {
		printf("FAILED: expected 'erich' got %s\n", cBuf);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");
	
	/* test: get strings */
	res = xmlconfig_get_string(cfg, "//config/section[@id='sec7']/topic/val", &ctable[0][0], STR_LEN, TABLE_SIZE);
	printf("xpath=//config/section[@id='sec7']/topic/val              ");
	if (res != 6) {
		printf("FAILED: expected 6 matches, got %i\n", res);
		exit(EXIT_FAILURE);
	}
	if (strcmp(ctable[0],"christian") != 0) {
		printf("FAILED: expected 'christian' got %s\n", ctable[0]);
		exit(EXIT_FAILURE);
	}
	if (strcmp(ctable[1],"olaf") != 0) {
		printf("FAILED: expected 'olaf' got %s\n", ctable[1]);
		exit(EXIT_FAILURE);
	}
	if (strcmp(ctable[2],"fridolin") != 0) {
		printf("FAILED: expected 'fridolin' got %s\n", ctable[2]);
		exit(EXIT_FAILURE);
	}
	if (strcmp(ctable[3],"wolfgang") != 0) {
		printf("FAILED: expected 'wolfgang' got %s\n", ctable[3]);
		exit(EXIT_FAILURE);
	}
	
	printf("OK\n");
	
	/* we are finished */
        xmlconfig_finish(cfg);

        return 0;
}

